#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <stdarg.h>

#include <jack/jack.h>
#include <jack/ringbuffer.h>

#include "rt_msg_buf.h"

struct rt_msg_buf {
        char *tmp_write;

        jack_ringbuffer_t *buffer;
        size_t size;

        unsigned int messages_lost;
        unsigned int messages_lost_reported;
};




RTMsgBuf *rt_msg_buf_new (size_t size)
{
        RTMsgBuf *this = malloc(sizeof(*this));

        if (this==NULL) return NULL;

        this->tmp_write = NULL;
        this->buffer = NULL;
        this->size = size;
        this->messages_lost = 0;
        this->messages_lost_reported = 0;

        this->buffer = jack_ringbuffer_create (size);

        this->tmp_write = malloc (size);

        jack_ringbuffer_mlock (this->buffer);

        if (this->buffer &&
            this->tmp_write) return this;
        else return rt_msg_buf_del(this);
}




RTMsgBuf *rt_msg_buf_del (RTMsgBuf *this)
{

        if (this==NULL) return NULL;

        jack_ringbuffer_free(this->buffer);

        free(this->tmp_write);

        free(this);

        return NULL;
}




void *rt_msg_buf_poll (RTMsgBuf *this, FILE *stream)
{

        jack_ringbuffer_data_t vec[2];

        jack_ringbuffer_get_read_vector (this->buffer, vec);

        for (int i = 0 ; i<2 ; i++)
                fwrite (vec[i].buf,sizeof(char),vec[i].len,stream);

        jack_ringbuffer_read_advance (this->buffer, vec[0].len + vec[1].len);

        unsigned int to_report=this->messages_lost-this->messages_lost_reported;


        if (to_report) {
                this->messages_lost_reported += to_report;
                fprintf(stream,"ERR: Lost %d messages.\n", to_report);
        }

        return NULL;
}




int rt_msg_buf_printf(RTMsgBuf *this, const char *format, ...)
{
        va_list ap;
        va_start (ap,format); /* format is last argument before ... */

        int return_value = vsnprintf(this->tmp_write, this->size, format, ap);
        va_end(ap);

        if (return_value<0) return return_value;

        size_t avail_data = (size_t) return_value;

        size_t avail_space = jack_ringbuffer_write_space (this->buffer);

        if (avail_space >= avail_data) {
                size_t written = jack_ringbuffer_write (this->buffer,
                                                        this->tmp_write,
                                                        avail_data);
                assert(written == avail_data);
                return written;
        } else {
                this->messages_lost++;
                return 0;
        }
}
