#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <limits.h>

#include "simple_src.h"



struct simple_src {
        jack_default_audio_sample_t buf1;
        jack_default_audio_sample_t buf2;
        long p;
        int first_sample;
};




SimpleSRC *simple_src_new()
{
        SimpleSRC *this = malloc(sizeof(*this));
        if (this==NULL) return NULL;
        simple_src_reset(this);
        return this;
}


void simple_src_reset(SimpleSRC *this)
{
        this->p = 0;
        this->buf1 = 0.0;
        this->buf2 = 0.0;
        this->first_sample = 1;
}

jack_nframes_t simple_src_process(SimpleSRC *this,
                                  jack_default_audio_sample_t *from,
                                  jack_nframes_t in_c,
                                  jack_default_audio_sample_t *to,
                                  jack_nframes_t buf_space,
                                  double r,
                                  jack_nframes_t *consumed,
                                  jack_nframes_t *overrun)
{
        assert(r<1.01);
        assert(r>0.99);

/*
        printf("WTF %p, %p %d, %p %d, %f\n",(void*)this,(void*)from,in_c,
                                            (void*)to,buf_space,r);
*/

        jack_default_audio_sample_t *toptr = to;

        unsigned long step = r * USHRT_MAX;

        if (!this->first_sample) {
                this->p += step;
        }
        this->first_sample = 0;

        while (((double)this->p/USHRT_MAX) < in_c - 1.0) {

                if (toptr >= to + buf_space) {
                        if (overrun) *overrun =
                               (  (in_c*USHRT_MAX)/r - this->p)/USHRT_MAX;

                        this->p -= step;

                        jack_nframes_t c;
                        if (this->p < 0) {
                                // this is ceil for -ve
                                // -15/10 = -1
                                c = this->p/USHRT_MAX;
                        } else {
                                // this is ceil for positive
                                // 15/10 = 1
                                // (15 - 1)/10 + 1 = 2
                                c = (this->p - 1)/USHRT_MAX + 1;
                        }

                        c += 1; // we have taken a step back


                        if (consumed) *consumed = c;

                        if (c == 0) {
                                this->buf1 = this->buf1;
                                this->buf2 = this->buf2;
                        } else if (c == 1) {
                                this->buf1 = this->buf2;
                                this->buf2 = from[c - 1];
                        } else {
                                this->buf1 = from[c - 2];
                                this->buf2 = from[c - 1];
                        }

                        this->p -= c * USHRT_MAX;

                        if (this->p > USHRT_MAX || this->p < -2.2*USHRT_MAX) {
                        printf("THIS SHOULD NEVER HAPPEN2 %ld %f\n",this->p,
                                (double)this->p/USHRT_MAX);
                        }

                        return toptr-to;
                }

                jack_default_audio_sample_t a,b;

                if (this->p < -2*USHRT_MAX) {
                        printf("THIS SHOULD NEVER HAPPEN %ld %f\n",this->p,
                                (double)this->p/USHRT_MAX);
                }
                //assert (this->p >= -2*USHRT_MAX);

                if (this->p < -USHRT_MAX) {
                        a = this->buf1;
                        b = this->buf2;
                } else if (this->p < 0) {
                        a = this->buf2;
                        b = from[0];
                } else {
                        a = from[this->p/USHRT_MAX];
                        b = from[this->p/USHRT_MAX+1];
                }

                *toptr++ = (((this->p + 2*USHRT_MAX)%USHRT_MAX) * (b-a)) /
                           USHRT_MAX + a;

                this->p += step;
        }

        this->p -= step;

        assert(this->p < ((signed long)in_c - 1)*USHRT_MAX);

        if (in_c == 0) {
                this->buf1 = this->buf1;
                this->buf2 = this->buf2;
        } else if (in_c == 1) {
                this->buf1 = this->buf2;
                this->buf2 = from[0];
        } else {
                this->buf1 = from[in_c - 2];
                this->buf2 = from[in_c - 1];
        }

        this->p -= in_c * USHRT_MAX;

        assert(this->p <= USHRT_MAX);
        assert(this->p >= -2.2 * USHRT_MAX);

        if (consumed) *consumed = in_c;

        return toptr-to;
}

void simple_src_del(SimpleSRC *this)
{
        free(this);
}

